/*
 * $Id: tree.prg 724 2011-09-20 22:34:09Z fperillo $
 */

/*
 * HMG Source Code
 *
 * Copyright 2002-2010 Roberto Lopez <mail.box.hmg@gmail.com>
 * http://sites.google.com/site/hmgweb/
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or( at your option ) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License FOR more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this software; see the file COPYING. IF not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA( or
 * visit the web site http://www.gnu.org/ ).
 *
 * As a special exception, you have permission FOR additional uses of the text
 * contained in this release of HMG.
 *
 * The exception is that, IF you link the HMG library with other
 * files to produce an executable, this does not by itself cause the resulting
 * executable to be covered by the GNU General Public License.
 * Your use of that executable is in no way restricted on account of linking the
 * HMG library code into it.
 */

#include "hbclass.ch"
#include "common.ch"
#include "hbqtgui.ch"

#define QTreeWidgetItem_ShowIndicator                    0  // The controls for expanding and collapsing will be shown for this item even if there are no children.
#define QTreeWidgetItem_DontShowIndicator                1  // The controls for expanding and collapsing will never be shown even if there are children. If the node is forced open the user will not be able to expand or collapse the item.
#define QTreeWidgetItem_DontShowIndicatorWhenChildless   2  // The controls for expanding and collapsing will be shown if the item contains children.

#define TREEITMTOPLV          0  // it means toplevel with QT
#define TREEITMCHILD          1  // it means child with QT

/*==============================================================================
   TREE class
   
   Note about aTreeItems array: this array store all the tree item (node and item).
      its structure is { Qtobject, Qtlevel }
      QtObject is QTreeWidgetItem
      QtLevel can be TREEITMTOPLV or TREEITMCHILD. I used these value to know - sometime and somewhere - if the Qtobject
      is a TopLevel item or a Child. Qt and HMG3 have difference about the type of items......
==============================================================================*/
CLASS TREE FROM CONTROL

   DATA cClass                                    INIT   "TREE"

// data: please preserve alfabetic order.
   DATA  aTreeItems                       INIT  {}    PROTECTED
   DATA  aItemImages                      INIT  {}    PROTECTED
   DATA  lItemIds                         INIT .F.    PROTECTED
   DATA  aNodeImages                      INIT  {}    PROTECTED
   DATA  nLastNode                        INIT  0     PROTECTED
   DATA  lRootButton                      INIT  .T.   PROTECTED

// method: please preserve alphabetic order. New and Create must be the first
   METHOD New
   METHOD Create
   METHOD AddItem
   METHOD AddNode
   METHOD AddTreeItem
   METHOD Animation                       SETGET
   METHOD Caption                         SETGET
   METHOD Collapse
   METHOD DeleteAllItems
   METHOD DeleteItem
   METHOD EndNode
   METHOD Expand
   METHOD Expanding                       SETGET
   METHOD HeaderItem                      SETGET
   METHOD Item                            SETGET
   METHOD ItemCount
   METHOD ItemIds                         SETGET
   METHOD ItemImages                      SETGET
   METHOD NodeImages                      SETGET
   METHOD OnChange                        SETGET         //overwrite standard method
   METHOD OnDblClick                      SETGET         //overwrite standard method
   METHOD OnEnter                         SETGET         //overwrite standard method
   METHOD Release
   METHOD RootButton                      SETGET
   METHOD Value                           SETGET
   METHOD __InsertItem                    PROTECTED
   METHOD __RemoveChildItems              PROTECTED
   METHOD __RefreshArray                  PROTECTED
   METHOD __RefreshAddChild               PROTECTED

// events handling
   METHOD __HmgConnectEv                  PROTECTED
   METHOD __HmgDisconnectEv               PROTECTED
   METHOD __HmgOnChangeExec               PROTECTED
   METHOD __HmgOnDblClickExec             PROTECTED
   METHOD __HmgOnEnterExec                PROTECTED

ENDCLASS

/*..............................................................................
   New
..............................................................................*/
METHOD New( cName, oParent ) CLASS TREE

   Super:New( cName, oParent )

   // create QT object
   IF hb_IsNil( ::oParent ) == .T.
      ::oQtObject := QTreeWidget()
   ELSE
      ::oQtObject := QTreeWidget( ::oParent:QtObject )
   ENDIF

   ::oQtFont     := ::oQTObject:font()

   ::oQtObject:setFocusPolicy( Qt_StrongFocus )
   ::oQTObject:setAutoFillBackground( .T. )

   ::HeaderItem( .T. )                                         // this don't show header. If you add a Caption to the QtreeWidget it will be .F.
   ::Expanding( .T. )                                          // This property holds whether to show controls for expanding and collapsing top-level items
   ::Animation( .T. )                                          // This property holds whether animations are enabled
   ::oQTObject:setColumnCount( 1 )

RETURN Self

/*..............................................................................
   Create
..............................................................................*/
METHOD Create() CLASS TREE

   IF ::lCreated == .T.
      RETURN NIL
   ENDIF

   ::lCreated := .T.

   // Create inner controls
   ::CreatePendingChildControls()

   ::__HmgConnectEv()

RETURN Self

/*..............................................................................
   AddItem
..............................................................................*/
METHOD AddItem( cCaption, nParent, aImages ) CLASS TREE

   LOCAL oHmgObject

   oHmgObject :=  TREEITEM():New( cCaption, aImages )

   IF hb_IsNumeric( nParent ) == .T.
      ::__InsertItem( oHmgObject, nParent, .F. )
   ELSE
      ::__InsertItem( oHmgObject, ::nLastNode, .F. )
   ENDIF

   ::__RefreshArray()

RETURN NIL

/*..............................................................................
   AddNode
..............................................................................*/
METHOD AddNode( cCaption, aImages ) CLASS TREE

   LOCAL oHmgObject

   aImages := IIF( hb_IsArray( aImages ) == .T., aImages, ::aNodeImages )

   oHmgObject :=  TREEITEM():New( cCaption, aImages )

   ::__InsertItem( oHmgObject, ::nLastNode, .T. )

   ::__RefreshArray()

RETURN NIL

/*..............................................................................
   AddTreeItem
..............................................................................*/
METHOD AddTreeItem( cCaption, aImages ) CLASS TREE

   LOCAL oHmgObject

   aImages := IIF( hb_IsArray( aImages ) == .T., aImages, ::aItemImages )

   oHmgObject :=  TREEITEM():New( cCaption, aImages )

   ::__InsertItem( oHmgObject, ::nLastNode, .F. )

   ::__RefreshArray()

RETURN NIL

/*..............................................................................
   Animation
   from Qt: This property holds whether animations are enabled
            If this property is true the treeview will animate expandsion and collasping of branches. 
            If this property is false, the treeview will expand or collapse branches immediately without showing the animation
..............................................................................*/
METHOD Animation( lArg1 ) CLASS TREE

   IF PCOUNT() == 0
      RETURN ::oQtObject:isAnimated()
   ELSEIF hb_IsLogical( lArg1 ) == .T.
      ::oQtObject:setAnimated( lArg1 )
   ENDIF

RETURN Self

/*..............................................................................
   Caption
..............................................................................*/
METHOD Caption( cArg1 ) CLASS TREE

   IF PCOUNT() == 0
      ::cCaption   := IIF( hb_IsChar( ::cCaption ) == .T., ::cCaption, "" )
      RETURN ::cCaption
   ELSEIF PCOUNT() == 1
      IF hb_IsChar( cArg1 ) == .T.
         ::cCaption := cArg1
         ::oQtObject:setHeaderLabel( ::cCaption )
         ::HeaderItem( .F. )
      ELSEIF hb_IsNil( cArg1 ) == .T.
         ::cCaption := ""
         ::oQtObject:setHeaderLabel( ::cCaption )
         ::HeaderItem( .T. )
      ENDIF
   ENDIF

RETURN Self

/*..............................................................................
   Collapse
..............................................................................*/
METHOD Collapse( nIndex ) CLASS TREE

   LOCAL oQtWidget

   IF hb_IsNumeric( nIndex ) == .T. .AND. nIndex > 0 .AND. nIndex <= LEN( ::aTreeItems )
      oQtWidget := ::aTreeItems[nIndex,1]
      ::oQTObject:collapseItem( oQtWidget )
   ENDIF
   
RETURN Self

/*..............................................................................
   DeleteAllItems
..............................................................................*/
METHOD DeleteAllItems() CLASS TREE

   ::aTreeItems := {}
   ::nLastNode  := 0
   ::oQTObject:clear()

RETURN Self

/*..............................................................................
   DeleteItem
..............................................................................*/
METHOD DeleteItem( nIndex ) CLASS TREE

   LOCAL oQtWidget, nX, oQtParent

   IF hb_IsNumeric( nIndex ) == .T. .AND. nIndex > 0 .AND. nIndex <= LEN( ::aTreeItems )
      oQtWidget := ::aTreeItems[nIndex,1]
      IF oQtWidget:childcount() > 0
         ::__RemoveChildItems( oQtWidget )
      ENDIF
      IF ::aTreeItems[nIndex,2] == TREEITMTOPLV   // 0 means TopLevel, 1 means Child
         nX := ::oQtObject:indexOfTopLevelItem( oQtWidget )
         ::oQtObject:takeTopLevelItem( nX )
      ELSE
         oQtParent := oQtWidget:parent()
         nX := oQtParent:indexOfChild( oQtWidget )
         oQtParent:takeChild( nX )
      ENDIF
      
      ::__RefreshArray()

   ENDIF

RETURN Self

/*..............................................................................
   EndNode
..............................................................................*/
METHOD EndNode() CLASS TREE

   LOCAL oQtWidget

   IF ::nLastNode > 0
      DO WHILE .T.
         ::nLastNode -= 1
         IF ::nLastNode == 0
            EXIT
         ELSE
            oQtWidget   := ::aTreeItems[::nLastNode,1]
            IF oQtWidget:childcount() > 0
               EXIT
            ENDIF
         ENDIF
      ENDDO
   ENDIF

RETURN Self

/*..............................................................................
   Expand
..............................................................................*/
METHOD Expand( nIndex ) CLASS TREE

   LOCAL oQtWidget

   IF hb_IsNumeric( nIndex ) == .T. .AND. nIndex > 0 .AND. nIndex <= LEN( ::aTreeItems )
      oQtWidget := ::aTreeItems[nIndex,1]
      ::oQTObject:expandItem( oQtWidget )
   ENDIF
   
RETURN Self

/*..............................................................................
   Expanding
   from Qt: This property holds whether to show controls for expanding and collapsing top-level items
..............................................................................*/
METHOD Expanding( lArg1 ) CLASS TREE

   IF PCOUNT() == 0
      RETURN ::oQtObject:rootIsDecorated()
   ELSEIF hb_IsLogical( lArg1 ) == .T.
      ::oQtObject:setRootIsDecorated( lArg1 )
   ENDIF

RETURN Self

/*..............................................................................
   HeaderItem
..............................................................................*/
METHOD HeaderItem( lArg1 ) CLASS TREE

   IF PCOUNT() == 0
      RETURN ::oQTObject:headerItem():isHidden()
   ELSEIF hb_IsLogical( lArg1 ) == .T.
      ::oQTObject:headerItem():setHidden( lArg1 )   // this don't show header: it will be enabled if you set Caption for QTreeWidget object
   ENDIF                                            // .T. is hidden, .F. is showed

RETURN Self

/*..............................................................................
   Item
..............................................................................*/
METHOD Item( nArg1, cArg1 ) CLASS TREE

   LOCAL oQtWidget

   IF PCOUNT() == 1 .AND. hb_IsNumeric( nArg1 ) .AND. nArg1 > 0 .AND. nArg1 <= LEN( ::aTreeItems )
      oQtWidget := ::aTreeItems[nArg1,1]
      RETURN oQtWidget:text( 0 )
   ENDIF

   IF PCOUNT() == 2 .AND. hb_IsNumeric( nArg1 ) .AND. nArg1 > 0 .AND. nArg1 <= LEN( ::aTreeItems ) .AND. hb_IsChar( cArg1 )
      oQtWidget := ::aTreeItems[nArg1,1]
      oQtWidget:setText( 0, cArg1 )
   ENDIF

RETURN Self

/*..............................................................................
   ItemCount
..............................................................................*/
METHOD ItemCount() CLASS TREE

RETURN LEN( ::aTreeItems )

/*..............................................................................
   ItemImages
..............................................................................*/
METHOD ItemImages( aArg1 ) CLASS TREE

   IF PCOUNT() == 0
      RETURN ::aItemImages
   ELSEIF hb_Isarray( aArg1 ) .AND. LEN( aArg1 ) <= 2
      ::aItemImages  := ACLONE( aArg1 )
   ENDIF

RETURN Self

/*..............................................................................
   ItemIds
..............................................................................*/
METHOD ItemIds( lArg1 ) CLASS TREE

   IF PCOUNT() == 0
      RETURN ::lItemIds
   ELSEIF hb_IsLogical( lArg1 ) == .T.
      ::lItemIds := lArg1
   ENDIF

RETURN NIL

/*..............................................................................
   NodeImages
..............................................................................*/
METHOD NodeImages( aArg1 ) CLASS TREE

   IF PCOUNT() == 0
      RETURN ::aNodeImages
   ELSEIF hb_Isarray( aArg1 ) .AND. LEN( aArg1 ) <= 2
      ::aNodeImages  := ACLONE( aArg1 )
   ENDIF

RETURN Self

/*..............................................................................
   OnChange
..............................................................................*/
METHOD OnChange( bValue ) CLASS TREE

   IF PCOUNT() == 0
      RETURN ::bOnChange
   ELSEIF PCOUNT() == 1
      IF hb_IsBlock( bValue ) == .T.
         ::bOnChange := bValue
         ::oQtObject:connect( "itemSelectionChanged()",  { || ::__HmgOnChangeExec() } )
      ELSEIF hb_IsNil( bValue ) == .T.
         ::bOnChange := NIL
         ::oQtObject:disconnect( "itemSelectionChanged()" )
      ENDIF
   ENDIF

RETURN Self

/*..............................................................................
   OnDblClick
..............................................................................*/
METHOD OnDblClick( bValue ) CLASS TREE

   IF PCOUNT() == 0
      RETURN ::bOnDblClick
   ELSEIF PCOUNT() == 1
      IF hb_IsBlock( bValue ) == .T.
         ::bOnDblClick := bValue
         ::oQtObject:connect( "itemDoubleClicked(QTreeWidgetItem*,int)", { || ::__HmgOnDblClickExec() } )
      ELSEIF hb_IsNil( bValue ) == .T.
         ::bOnDblClick := NIL
         ::oQtObject:disconnect( "itemDoubleClicked(QTreeWidgetItem*,int)" )
      ENDIF
   ENDIF

RETURN Self

/*..............................................................................
   OnEnter
..............................................................................*/
METHOD OnEnter( bValue ) CLASS TREE

   IF PCOUNT() == 0
      RETURN ::bOnEnter
   ELSEIF PCOUNT() == 1
      IF hb_IsBlock( bValue ) == .T.
         ::bOnEnter := bValue
         ::oQtObject:connect( "itemActivated(QTreeWidgetItem*,int)",  { || ::__HmgOnEnterExec() } )
      ELSEIF hb_IsNil( bValue ) == .T.
         ::bOnEnter := NIL
         ::oQtObject:disconnect( "itemActivated(QTreeWidgetItem*,int)" )
      ENDIF
   ENDIF

RETURN Self

/*..............................................................................
   Release
..............................................................................*/
METHOD Release() CLASS TREE

//#MOD20110915_001   LOCAL cName, oObject, aChildren

/*#MOD20110915_001
   // before call release (qt:close) for every children
   aChildren   := ::ChildItemList()
   FOR EACH cName IN aChildren:Keys
      IF hb_IsObject( oObject := aChildren[cName] ) == .T.     // the object it's not remove from stack but can be "U"
         IF __objHasMethod( oObject, "Release" ) == .T.
            oObject:Release()
         ENDIF
      ENDIF
   NEXT
*/

   // remove current HMG object from parent stack
   IF hb_IsNil( ::oParent ) == .F.
      ::oParent:ChildRemove( ::cName )
   ENDIF

//#MOD20110915_001   ::__HmgDisconnectEv()

RETURN Self

/*..............................................................................
   RootButton
..............................................................................*/
METHOD RootButton( lArg1 ) CLASS TREE

   LOCAL oQtWidget
   
   IF PCOUNT() == 0
      RETURN ::lRootButton
   ELSEIF hb_IsLogical( lArg1 ) == .T.
      ::lRootButton  := lArg1
      IF LEN( ::aTreeItems ) > 0
         oQtWidget := ::aTreeItems[1,1]
         IF ::lRootButton == .T.
            oQtWidget:setChildIndicatorPolicy( QTreeWidgetItem_DontShowIndicatorWhenChildless )
         ELSE
            oQtWidget:setChildIndicatorPolicy( QTreeWidgetItem_DontShowIndicator )
         ENDIF
      ENDIF
   ENDIF

RETURN Self

/*..............................................................................
   Value
..............................................................................*/
METHOD Value( nArg1 ) CLASS TREE

   LOCAL oQtWidget, nX, nSelected := 0

   IF PCOUNT() == 0
      FOR nX := 1 TO LEN( ::aTreeItems )
         oQtWidget   := ::aTreeItems[nX,1]
         IF oQtWidget:isSelected()
            nSelected   := nX
         ENDIF
      NEXT nX
      RETURN nSelected
   ELSEIF hb_IsNumeric( nArg1 ) == .T.
      IF nArg1 > 0 .AND. nArg1 <= LEN( ::aTreeItems )
         oQtWidget   := ::aTreeItems[nX,1]
         ::oQtObject:setCurrentItem( oQtWidget )
      ENDIF
   ENDIF

RETURN Self

/*..............................................................................
   __InsertItem
..............................................................................*/
METHOD __InsertItem( oHmgObject, nParent, lForceNode ) CLASS TREE

   LOCAL oQtParent

   IF nParent == 0 .OR. nParent > LEN( ::aTreeItems )
      ::oQtObject:addTopLevelItem( oHmgObject:QtObject )
      AADD( ::aTreeItems, { oHmgObject:QtObject, TREEITMTOPLV } )  // 0 means TopLevel, 1 means Child
      IF hb_IsLogical( lForceNode ) == .T. .AND. lForceNode == .T.
         ::nLastNode := LEN( ::aTreeItems )
      ENDIF
   ELSE
      oQtParent  := ::aTreeItems[nParent,1]
      oQtParent:addChild( oHmgObject:QtObject )
      IF nParent < LEN( ::aTreeItems )
         ::aTreeItems := hb_aIns( ::aTreeItems, nParent + 1, { oHmgObject:QtObject, TREEITMCHILD } ) // 0 means TopLevel, 1 means Child
         IF hb_IsLogical( lForceNode ) == .T. .AND. lForceNode == .T.
            ::nLastNode := nParent + 2
         ENDIF
      ELSE
         AADD( ::aTreeItems, { oHmgObject:QtObject, TREEITMCHILD } )  // 0 means TopLevel, 1 means Child
         IF hb_IsLogical( lForceNode ) == .T. .AND. lForceNode == .T.
            ::nLastNode := LEN( ::aTreeItems )
         ENDIF
      ENDIF
   ENDIF

RETURN NIL

/*..............................................................................
   __RefreshArray
..............................................................................*/
METHOD __RefreshArray() CLASS TREE

   LOCAL nX, oQtWidget
   
   ::aTreeItems := {}
   FOR nX := 1 TO ::oQtObject:topLevelItemCount()
      oQtWidget := ::oQtObject:toplevelitem( nX-1 )
      IF nX == 1
         IF ::lRootButton == .T.
            oQtWidget:setChildIndicatorPolicy( QTreeWidgetItem_DontShowIndicatorWhenChildless )
         ELSE
            oQtWidget:setChildIndicatorPolicy( QTreeWidgetItem_DontShowIndicator )
         ENDIF
      ENDIF
      ::__RefreshAddChild( oQtWidget, TREEITMTOPLV ) // 0 means TopLevel, 1 means Child
   NEXT nX

RETURN NIL

/*..............................................................................
   __RefreshArray
..............................................................................*/
METHOD __RefreshAddChild( oQtWidget, nType ) CLASS TREE

   LOCAL nX, oQtChild

   AADD( ::aTreeItems, { oQtWidget, nType } ) // 0 means TopLevel, 1 means Child
   IF oQtWidget:childcount() > 0
     FOR nX := 1 TO oQtWidget:childcount()
        oQtChild := oQtWidget:child( nX-1 )
        ::__RefreshAddChild( oQtChild, TREEITMCHILD ) // 0 means TopLevel, 1 means Child
     NEXT nX
   ENDIF

RETURN NIL

/*..............................................................................
   __RemoveChildItems
..............................................................................*/
METHOD __RemoveChildItems( oQtWidget ) CLASS TREE

   LOCAL nX, oQtChild, oQtParent

   IF oQtWidget:childcount() > 0
     FOR nX := 1 TO oQtWidget:childcount()
        oQtChild := oQtWidget:child( nX-1 )
        ::__RemoveChildItems( oQtChild )
     NEXT nX
   ENDIF

   oQtParent := oQtWidget:parent()
   nX := oQtParent:indexOfChild( oQtWidget )
   oQtParent:takeChild( nX )

RETURN NIL

/*..............................................................................
   __HmgConnectEv
..............................................................................*/
METHOD __HmgConnectEv() CLASS TREE

   ::oQtObject:connect( QEvent_Close,             { |oEv| ::__HmgOnCloseExec(oEv) } )
//see overwrite OnChange method   ::oQtObject:connect( "itemSelectionChanged()",  { || ::__HmgOnChangeExec() } )
//see overwrite OnDblClick method   ::oQtObject:connect( "itemDoubleClicked(QTreeWidgetItem*,int)", { || ::__HmgOnDblClickExec() } )
//see overwrite OnDblClick method   ::oQtObject:connect( "itemActivated(QTreeWidgetItem*,int)",  { || ::__HmgOnEnterExec() } )

RETURN Self

/*..............................................................................
   __HmgDisconnectEv
..............................................................................*/
METHOD __HmgDisconnectEv() CLASS TREE

   ::oQtObject:disconnect( QEvent_Close )
   ::oQtObject:disconnect( "itemSelectionChanged()" )
   ::oQtObject:disconnect( "itemDoubleClicked(QTreeWidgetItem*,int)" )
   ::oQtObject:disconnect( "itemActivated(QTreeWidgetItem*,int)" )

RETURN Self

/*..............................................................................
   __HmgOnChangeExec
..............................................................................*/
METHOD __HmgOnChangeExec()  CLASS TREE

   LOCAL lQtEventStop   := .F.      // .F. means don't stop event hanlder, else .T. STOP

   IF hb_IsBlock( ::bOnChange ) == .T.
      EVAL( ::bOnChange )
   ENDIF

RETURN lQtEventStop

/*..............................................................................
   __HmgOnDblClickExec
..............................................................................*/
METHOD __HmgOnDblClickExec()  CLASS TREE

   LOCAL lQtEventStop   := .F.      // .F. means don't stop event hanlder, else .T. STOP

   IF hb_IsBlock( ::bOnDblClick ) == .T.
      EVAL( ::bOnDblClick )
   ENDIF

RETURN lQtEventStop

/*..............................................................................
   __HmgOnEnterExec
..............................................................................*/
METHOD __HmgOnEnterExec()  CLASS TREE

   LOCAL lQtEventStop   := .F.      // .F. means don't stop event hanlder, else .T. STOP

   IF hb_IsBlock( ::bOnEnter ) == .T.
      EVAL( ::bOnEnter )
   ENDIF

RETURN lQtEventStop

/*==============================================================================
 TREELAY class
 layer class for TREEitems (Node and Item)

 see:
 http://www.java2s.com/Code/Cpp/Qt/UsingQTreeView.htm
 http://doc.qt.nokia.com/stable/qtreewidgetitem.html
 http://doc.qt.nokia.com/stable/qtreewidget.html

==============================================================================*/
CLASS TREEITEM

   DATA  cObjClassName                    INIT  "TREELAY"

// data: please preserve alfabetic order.
   DATA  oIcon                            INIT  NIL               PROTECTED
   DATA  aImages                          INIT  {}                PROTECTED
   DATA  oQtObject                        INIT  NIL               PROTECTED

// method: please preserve alphabetic order. New and Create must be the first
   METHOD New
   METHOD Caption                         SETGET
   METHOD ItemImages                      SETGET
   METHOD QtObject

// events handling

ENDCLASS

/*..............................................................................
   New
..............................................................................*/
METHOD New( cCaption, aImages ) CLASS TREEITEM

   ::oQtObject := QTreeWidgetItem()
   
   ::oIcon  := QIcon()

   IF hb_IsChar( cCaption ) == .T.
      ::Caption   := cCaption
   ENDIF
   IF hb_IsArray( aImages ) == .T.
      ::ItemImages   := aImages
   ENDIF

RETURN Self

/*..............................................................................
   Caption
..............................................................................*/
METHOD Caption( cArg1 ) CLASS TREEITEM

   IF PCOUNT() == 0
      RETURN ::oQtObject:text( 0 )
   ELSEIF PCOUNT() == 1
      ::oQtObject:setText( 0, cArg1 )
   ENDIF

RETURN Self

/*..............................................................................
   ItemImages
   TODO: remove/disable icons
..............................................................................*/
METHOD ItemImages( aArg1 ) CLASS TREEITEM

   LOCAL oNewIcon

   IF PCOUNT() == 0
      RETURN ::aImages
   ELSEIF hb_IsArray( aArg1 ) == .T.
      IF LEN( aArg1 ) >= 1
         ::aImages  := aArg1
         oNewIcon := QPixmap( ::aImages[1] )
         ::oIcon:addPixmap( oNewIcon, QIcon_Normal, QIcon_On )
         ::oQtObject:setIcon( 0, ::oIcon )
         IF LEN( ::aImages ) == 2
            oNewIcon := QPixmap( ::aImages[2] )
            ::oIcon:addPixmap( oNewIcon, QIcon_Selected, QIcon_On )
            ::oQtObject:setIcon( 0, ::oIcon )
         ENDIF
      ENDIF
   ENDIF

RETURN Self

/*..............................................................................
   QtObject
..............................................................................*/
METHOD QtObject() CLASS TREEITEM
RETURN ::oQtObject